@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2020 Google LLC
+# Copyright 2022 Google LLC
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -16,16 +16,21 @@
 from collections import OrderedDict
 import functools
 import re
-from typing import Dict, Sequence, Tuple, Type, Union
+from typing import Dict, Mapping, Optional, Sequence, Tuple, Type, Union
 import pkg_resources
 
-import google.api_core.client_options as ClientOptions  # type: ignore
-from google.api_core import exceptions as core_exceptions  # type: ignore
-from google.api_core import gapic_v1  # type: ignore
-from google.api_core import retry as retries  # type: ignore
+from google.api_core.client_options import ClientOptions
+from google.api_core import exceptions as core_exceptions
+from google.api_core import gapic_v1
+from google.api_core import retry as retries
 from google.auth import credentials as ga_credentials  # type: ignore
 from google.oauth2 import service_account  # type: ignore
 
+try:
+    OptionalRetry = Union[retries.Retry, gapic_v1.method._MethodDefault]
+except AttributeError:  # pragma: NO COVER
+    OptionalRetry = Union[retries.Retry, object]  # type: ignore
+
 from google.api_core import operation  # type: ignore
 from google.api_core import operation_async  # type: ignore
 from google.cloud.recommendationengine_v1beta1.services.catalog_service import pagers
@@ -110,6 +115,42 @@ class CatalogServiceAsyncClient:
 
     from_service_account_json = from_service_account_file
 
+    @classmethod
+    def get_mtls_endpoint_and_cert_source(
+        cls, client_options: Optional[ClientOptions] = None
+    ):
+        """Return the API endpoint and client cert source for mutual TLS.
+
+        The client cert source is determined in the following order:
+        (1) if `GOOGLE_API_USE_CLIENT_CERTIFICATE` environment variable is not "true", the
+        client cert source is None.
+        (2) if `client_options.client_cert_source` is provided, use the provided one; if the
+        default client cert source exists, use the default one; otherwise the client cert
+        source is None.
+
+        The API endpoint is determined in the following order:
+        (1) if `client_options.api_endpoint` if provided, use the provided one.
+        (2) if `GOOGLE_API_USE_CLIENT_CERTIFICATE` environment variable is "always", use the
+        default mTLS endpoint; if the environment variabel is "never", use the default API
+        endpoint; otherwise if client cert source exists, use the default mTLS endpoint, otherwise
+        use the default API endpoint.
+
+        More details can be found at https://google.aip.dev/auth/4114.
+
+        Args:
+            client_options (google.api_core.client_options.ClientOptions): Custom options for the
+                client. Only the `api_endpoint` and `client_cert_source` properties may be used
+                in this method.
+
+        Returns:
+            Tuple[str, Callable[[], Tuple[bytes, bytes]]]: returns the API endpoint and the
+                client cert source to use.
+
+        Raises:
+            google.auth.exceptions.MutualTLSChannelError: If any errors happen.
+        """
+        return CatalogServiceClient.get_mtls_endpoint_and_cert_source(client_options)  # type: ignore
+
     @property
     def transport(self) -> CatalogServiceTransport:
         """Returns the transport used by the client instance.
@@ -172,18 +213,43 @@ class CatalogServiceAsyncClient:
 
     async def create_catalog_item(
         self,
-        request: catalog_service.CreateCatalogItemRequest = None,
+        request: Union[catalog_service.CreateCatalogItemRequest, dict] = None,
         *,
         parent: str = None,
         catalog_item: catalog.CatalogItem = None,
-        retry: retries.Retry = gapic_v1.method.DEFAULT,
+        retry: OptionalRetry = gapic_v1.method.DEFAULT,
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> catalog.CatalogItem:
         r"""Creates a catalog item.
 
+        .. code-block:: python
+
+            from google.cloud import recommendationengine_v1beta1
+
+            async def sample_create_catalog_item():
+                # Create a client
+                client = recommendationengine_v1beta1.CatalogServiceAsyncClient()
+
+                # Initialize request argument(s)
+                catalog_item = recommendationengine_v1beta1.CatalogItem()
+                catalog_item.id = "id_value"
+                catalog_item.category_hierarchies.categories = ['categories_value_1', 'categories_value_2']
+                catalog_item.title = "title_value"
+
+                request = recommendationengine_v1beta1.CreateCatalogItemRequest(
+                    parent="parent_value",
+                    catalog_item=catalog_item,
+                )
+
+                # Make the request
+                response = await client.create_catalog_item(request=request)
+
+                # Handle the response
+                print(response)
+
         Args:
-            request (:class:`google.cloud.recommendationengine_v1beta1.types.CreateCatalogItemRequest`):
+            request (Union[google.cloud.recommendationengine_v1beta1.types.CreateCatalogItemRequest, dict]):
                 The request object. Request message for
                 CreateCatalogItem method.
             parent (:class:`str`):
@@ -211,7 +277,7 @@ class CatalogServiceAsyncClient:
 
         """
         # Create or coerce a protobuf request object.
-        # Sanity check: If we got a request object, we should *not* have
+        # Quick check: If we got a request object, we should *not* have
         # gotten any keyword arguments that map to the request.
         has_flattened_params = any([parent, catalog_item])
         if request is not None and has_flattened_params:
@@ -254,24 +320,48 @@ class CatalogServiceAsyncClient:
         )
 
         # Send the request.
-        response = await rpc(request, retry=retry, timeout=timeout, metadata=metadata,)
+        response = await rpc(
+            request,
+            retry=retry,
+            timeout=timeout,
+            metadata=metadata,
+        )
 
         # Done; return the response.
         return response
 
     async def get_catalog_item(
         self,
-        request: catalog_service.GetCatalogItemRequest = None,
+        request: Union[catalog_service.GetCatalogItemRequest, dict] = None,
         *,
         name: str = None,
-        retry: retries.Retry = gapic_v1.method.DEFAULT,
+        retry: OptionalRetry = gapic_v1.method.DEFAULT,
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> catalog.CatalogItem:
         r"""Gets a specific catalog item.
 
+        .. code-block:: python
+
+            from google.cloud import recommendationengine_v1beta1
+
+            async def sample_get_catalog_item():
+                # Create a client
+                client = recommendationengine_v1beta1.CatalogServiceAsyncClient()
+
+                # Initialize request argument(s)
+                request = recommendationengine_v1beta1.GetCatalogItemRequest(
+                    name="name_value",
+                )
+
+                # Make the request
+                response = await client.get_catalog_item(request=request)
+
+                # Handle the response
+                print(response)
+
         Args:
-            request (:class:`google.cloud.recommendationengine_v1beta1.types.GetCatalogItemRequest`):
+            request (Union[google.cloud.recommendationengine_v1beta1.types.GetCatalogItemRequest, dict]):
                 The request object. Request message for GetCatalogItem
                 method.
             name (:class:`str`):
@@ -294,7 +384,7 @@ class CatalogServiceAsyncClient:
 
         """
         # Create or coerce a protobuf request object.
-        # Sanity check: If we got a request object, we should *not* have
+        # Quick check: If we got a request object, we should *not* have
         # gotten any keyword arguments that map to the request.
         has_flattened_params = any([name])
         if request is not None and has_flattened_params:
@@ -335,25 +425,50 @@ class CatalogServiceAsyncClient:
         )
 
         # Send the request.
-        response = await rpc(request, retry=retry, timeout=timeout, metadata=metadata,)
+        response = await rpc(
+            request,
+            retry=retry,
+            timeout=timeout,
+            metadata=metadata,
+        )
 
         # Done; return the response.
         return response
 
     async def list_catalog_items(
         self,
-        request: catalog_service.ListCatalogItemsRequest = None,
+        request: Union[catalog_service.ListCatalogItemsRequest, dict] = None,
         *,
         parent: str = None,
         filter: str = None,
-        retry: retries.Retry = gapic_v1.method.DEFAULT,
+        retry: OptionalRetry = gapic_v1.method.DEFAULT,
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> pagers.ListCatalogItemsAsyncPager:
         r"""Gets a list of catalog items.
 
+        .. code-block:: python
+
+            from google.cloud import recommendationengine_v1beta1
+
+            async def sample_list_catalog_items():
+                # Create a client
+                client = recommendationengine_v1beta1.CatalogServiceAsyncClient()
+
+                # Initialize request argument(s)
+                request = recommendationengine_v1beta1.ListCatalogItemsRequest(
+                    parent="parent_value",
+                )
+
+                # Make the request
+                page_result = client.list_catalog_items(request=request)
+
+                # Handle the response
+                async for response in page_result:
+                    print(response)
+
         Args:
-            request (:class:`google.cloud.recommendationengine_v1beta1.types.ListCatalogItemsRequest`):
+            request (Union[google.cloud.recommendationengine_v1beta1.types.ListCatalogItemsRequest, dict]):
                 The request object. Request message for ListCatalogItems
                 method.
             parent (:class:`str`):
@@ -386,7 +501,7 @@ class CatalogServiceAsyncClient:
 
         """
         # Create or coerce a protobuf request object.
-        # Sanity check: If we got a request object, we should *not* have
+        # Quick check: If we got a request object, we should *not* have
         # gotten any keyword arguments that map to the request.
         has_flattened_params = any([parent, filter])
         if request is not None and has_flattened_params:
@@ -429,12 +544,20 @@ class CatalogServiceAsyncClient:
         )
 
         # Send the request.
-        response = await rpc(request, retry=retry, timeout=timeout, metadata=metadata,)
+        response = await rpc(
+            request,
+            retry=retry,
+            timeout=timeout,
+            metadata=metadata,
+        )
 
         # This method is paged; wrap the response in a pager, which provides
         # an `__aiter__` convenience method.
         response = pagers.ListCatalogItemsAsyncPager(
-            method=rpc, request=request, response=response, metadata=metadata,
+            method=rpc,
+            request=request,
+            response=response,
+            metadata=metadata,
         )
 
         # Done; return the response.
@@ -442,20 +565,45 @@ class CatalogServiceAsyncClient:
 
     async def update_catalog_item(
         self,
-        request: catalog_service.UpdateCatalogItemRequest = None,
+        request: Union[catalog_service.UpdateCatalogItemRequest, dict] = None,
         *,
         name: str = None,
         catalog_item: catalog.CatalogItem = None,
         update_mask: field_mask_pb2.FieldMask = None,
-        retry: retries.Retry = gapic_v1.method.DEFAULT,
+        retry: OptionalRetry = gapic_v1.method.DEFAULT,
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> catalog.CatalogItem:
         r"""Updates a catalog item. Partial updating is
         supported. Non-existing items will be created.
 
+        .. code-block:: python
+
+            from google.cloud import recommendationengine_v1beta1
+
+            async def sample_update_catalog_item():
+                # Create a client
+                client = recommendationengine_v1beta1.CatalogServiceAsyncClient()
+
+                # Initialize request argument(s)
+                catalog_item = recommendationengine_v1beta1.CatalogItem()
+                catalog_item.id = "id_value"
+                catalog_item.category_hierarchies.categories = ['categories_value_1', 'categories_value_2']
+                catalog_item.title = "title_value"
+
+                request = recommendationengine_v1beta1.UpdateCatalogItemRequest(
+                    name="name_value",
+                    catalog_item=catalog_item,
+                )
+
+                # Make the request
+                response = await client.update_catalog_item(request=request)
+
+                # Handle the response
+                print(response)
+
         Args:
-            request (:class:`google.cloud.recommendationengine_v1beta1.types.UpdateCatalogItemRequest`):
+            request (Union[google.cloud.recommendationengine_v1beta1.types.UpdateCatalogItemRequest, dict]):
                 The request object. Request message for
                 UpdateCatalogItem method.
             name (:class:`str`):
@@ -493,7 +641,7 @@ class CatalogServiceAsyncClient:
 
         """
         # Create or coerce a protobuf request object.
-        # Sanity check: If we got a request object, we should *not* have
+        # Quick check: If we got a request object, we should *not* have
         # gotten any keyword arguments that map to the request.
         has_flattened_params = any([name, catalog_item, update_mask])
         if request is not None and has_flattened_params:
@@ -538,24 +686,45 @@ class CatalogServiceAsyncClient:
         )
 
         # Send the request.
-        response = await rpc(request, retry=retry, timeout=timeout, metadata=metadata,)
+        response = await rpc(
+            request,
+            retry=retry,
+            timeout=timeout,
+            metadata=metadata,
+        )
 
         # Done; return the response.
         return response
 
     async def delete_catalog_item(
         self,
-        request: catalog_service.DeleteCatalogItemRequest = None,
+        request: Union[catalog_service.DeleteCatalogItemRequest, dict] = None,
         *,
         name: str = None,
-        retry: retries.Retry = gapic_v1.method.DEFAULT,
+        retry: OptionalRetry = gapic_v1.method.DEFAULT,
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> None:
         r"""Deletes a catalog item.
 
+        .. code-block:: python
+
+            from google.cloud import recommendationengine_v1beta1
+
+            async def sample_delete_catalog_item():
+                # Create a client
+                client = recommendationengine_v1beta1.CatalogServiceAsyncClient()
+
+                # Initialize request argument(s)
+                request = recommendationengine_v1beta1.DeleteCatalogItemRequest(
+                    name="name_value",
+                )
+
+                # Make the request
+                await client.delete_catalog_item(request=request)
+
         Args:
-            request (:class:`google.cloud.recommendationengine_v1beta1.types.DeleteCatalogItemRequest`):
+            request (Union[google.cloud.recommendationengine_v1beta1.types.DeleteCatalogItemRequest, dict]):
                 The request object. Request message for
                 DeleteCatalogItem method.
             name (:class:`str`):
@@ -572,7 +741,7 @@ class CatalogServiceAsyncClient:
                 sent along with the request as metadata.
         """
         # Create or coerce a protobuf request object.
-        # Sanity check: If we got a request object, we should *not* have
+        # Quick check: If we got a request object, we should *not* have
         # gotten any keyword arguments that map to the request.
         has_flattened_params = any([name])
         if request is not None and has_flattened_params:
@@ -614,18 +783,21 @@ class CatalogServiceAsyncClient:
 
         # Send the request.
         await rpc(
-            request, retry=retry, timeout=timeout, metadata=metadata,
+            request,
+            retry=retry,
+            timeout=timeout,
+            metadata=metadata,
         )
 
     async def import_catalog_items(
         self,
-        request: import_.ImportCatalogItemsRequest = None,
+        request: Union[import_.ImportCatalogItemsRequest, dict] = None,
         *,
         parent: str = None,
         request_id: str = None,
         input_config: import_.InputConfig = None,
         errors_config: import_.ImportErrorsConfig = None,
-        retry: retries.Retry = gapic_v1.method.DEFAULT,
+        retry: OptionalRetry = gapic_v1.method.DEFAULT,
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> operation_async.AsyncOperation:
@@ -637,8 +809,31 @@ class CatalogServiceAsyncClient:
         it is possible for a subset of the items to be
         successfully updated.
 
+        .. code-block:: python
+
+            from google.cloud import recommendationengine_v1beta1
+
+            async def sample_import_catalog_items():
+                # Create a client
+                client = recommendationengine_v1beta1.CatalogServiceAsyncClient()
+
+                # Initialize request argument(s)
+                request = recommendationengine_v1beta1.ImportCatalogItemsRequest(
+                    parent="parent_value",
+                )
+
+                # Make the request
+                operation = client.import_catalog_items(request=request)
+
+                print("Waiting for operation to complete...")
+
+                response = await operation.result()
+
+                # Handle the response
+                print(response)
+
         Args:
-            request (:class:`google.cloud.recommendationengine_v1beta1.types.ImportCatalogItemsRequest`):
+            request (Union[google.cloud.recommendationengine_v1beta1.types.ImportCatalogItemsRequest, dict]):
                 The request object. Request message for Import methods.
             parent (:class:`str`):
                 Required.
@@ -691,7 +886,7 @@ class CatalogServiceAsyncClient:
 
         """
         # Create or coerce a protobuf request object.
-        # Sanity check: If we got a request object, we should *not* have
+        # Quick check: If we got a request object, we should *not* have
         # gotten any keyword arguments that map to the request.
         has_flattened_params = any([parent, request_id, input_config, errors_config])
         if request is not None and has_flattened_params:
@@ -738,7 +933,12 @@ class CatalogServiceAsyncClient:
         )
 
         # Send the request.
-        response = await rpc(request, retry=retry, timeout=timeout, metadata=metadata,)
+        response = await rpc(
+            request,
+            retry=retry,
+            timeout=timeout,
+            metadata=metadata,
+        )
 
         # Wrap the response in an operation future.
         response = operation_async.from_gapic(
