@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2020 Google LLC
+# Copyright 2022 Google LLC
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -14,22 +14,26 @@
 # limitations under the License.
 #
 from collections import OrderedDict
-from distutils import util
 import os
 import re
-from typing import Dict, Optional, Sequence, Tuple, Type, Union
+from typing import Dict, Mapping, Optional, Sequence, Tuple, Type, Union
 import pkg_resources
 
-from google.api_core import client_options as client_options_lib  # type: ignore
-from google.api_core import exceptions as core_exceptions  # type: ignore
-from google.api_core import gapic_v1  # type: ignore
-from google.api_core import retry as retries  # type: ignore
+from google.api_core import client_options as client_options_lib
+from google.api_core import exceptions as core_exceptions
+from google.api_core import gapic_v1
+from google.api_core import retry as retries
 from google.auth import credentials as ga_credentials  # type: ignore
 from google.auth.transport import mtls  # type: ignore
 from google.auth.transport.grpc import SslCredentials  # type: ignore
 from google.auth.exceptions import MutualTLSChannelError  # type: ignore
 from google.oauth2 import service_account  # type: ignore
 
+try:
+    OptionalRetry = Union[retries.Retry, gapic_v1.method._MethodDefault]
+except AttributeError:  # pragma: NO COVER
+    OptionalRetry = Union[retries.Retry, object]  # type: ignore
+
 from google.api_core import operation  # type: ignore
 from google.api_core import operation_async  # type: ignore
 from google.cloud.recommendationengine_v1beta1.services.catalog_service import pagers
@@ -57,7 +61,10 @@ class CatalogServiceClientMeta(type):
     _transport_registry["grpc"] = CatalogServiceGrpcTransport
     _transport_registry["grpc_asyncio"] = CatalogServiceGrpcAsyncIOTransport
 
-    def get_transport_class(cls, label: str = None,) -> Type[CatalogServiceTransport]:
+    def get_transport_class(
+        cls,
+        label: str = None,
+    ) -> Type[CatalogServiceTransport]:
         """Returns an appropriate transport class.
 
         Args:
@@ -164,10 +171,16 @@ class CatalogServiceClient(metaclass=CatalogServiceClientMeta):
         return self._transport
 
     @staticmethod
-    def catalog_path(project: str, location: str, catalog: str,) -> str:
+    def catalog_path(
+        project: str,
+        location: str,
+        catalog: str,
+    ) -> str:
         """Returns a fully-qualified catalog string."""
         return "projects/{project}/locations/{location}/catalogs/{catalog}".format(
-            project=project, location=location, catalog=catalog,
+            project=project,
+            location=location,
+            catalog=catalog,
         )
 
     @staticmethod
@@ -180,7 +193,9 @@ class CatalogServiceClient(metaclass=CatalogServiceClientMeta):
         return m.groupdict() if m else {}
 
     @staticmethod
-    def common_billing_account_path(billing_account: str,) -> str:
+    def common_billing_account_path(
+        billing_account: str,
+    ) -> str:
         """Returns a fully-qualified billing_account string."""
         return "billingAccounts/{billing_account}".format(
             billing_account=billing_account,
@@ -193,9 +208,13 @@ class CatalogServiceClient(metaclass=CatalogServiceClientMeta):
         return m.groupdict() if m else {}
 
     @staticmethod
-    def common_folder_path(folder: str,) -> str:
+    def common_folder_path(
+        folder: str,
+    ) -> str:
         """Returns a fully-qualified folder string."""
-        return "folders/{folder}".format(folder=folder,)
+        return "folders/{folder}".format(
+            folder=folder,
+        )
 
     @staticmethod
     def parse_common_folder_path(path: str) -> Dict[str, str]:
@@ -204,9 +223,13 @@ class CatalogServiceClient(metaclass=CatalogServiceClientMeta):
         return m.groupdict() if m else {}
 
     @staticmethod
-    def common_organization_path(organization: str,) -> str:
+    def common_organization_path(
+        organization: str,
+    ) -> str:
         """Returns a fully-qualified organization string."""
-        return "organizations/{organization}".format(organization=organization,)
+        return "organizations/{organization}".format(
+            organization=organization,
+        )
 
     @staticmethod
     def parse_common_organization_path(path: str) -> Dict[str, str]:
@@ -215,9 +238,13 @@ class CatalogServiceClient(metaclass=CatalogServiceClientMeta):
         return m.groupdict() if m else {}
 
     @staticmethod
-    def common_project_path(project: str,) -> str:
+    def common_project_path(
+        project: str,
+    ) -> str:
         """Returns a fully-qualified project string."""
-        return "projects/{project}".format(project=project,)
+        return "projects/{project}".format(
+            project=project,
+        )
 
     @staticmethod
     def parse_common_project_path(path: str) -> Dict[str, str]:
@@ -226,10 +253,14 @@ class CatalogServiceClient(metaclass=CatalogServiceClientMeta):
         return m.groupdict() if m else {}
 
     @staticmethod
-    def common_location_path(project: str, location: str,) -> str:
+    def common_location_path(
+        project: str,
+        location: str,
+    ) -> str:
         """Returns a fully-qualified location string."""
         return "projects/{project}/locations/{location}".format(
-            project=project, location=location,
+            project=project,
+            location=location,
         )
 
     @staticmethod
@@ -238,6 +269,73 @@ class CatalogServiceClient(metaclass=CatalogServiceClientMeta):
         m = re.match(r"^projects/(?P<project>.+?)/locations/(?P<location>.+?)$", path)
         return m.groupdict() if m else {}
 
+    @classmethod
+    def get_mtls_endpoint_and_cert_source(
+        cls, client_options: Optional[client_options_lib.ClientOptions] = None
+    ):
+        """Return the API endpoint and client cert source for mutual TLS.
+
+        The client cert source is determined in the following order:
+        (1) if `GOOGLE_API_USE_CLIENT_CERTIFICATE` environment variable is not "true", the
+        client cert source is None.
+        (2) if `client_options.client_cert_source` is provided, use the provided one; if the
+        default client cert source exists, use the default one; otherwise the client cert
+        source is None.
+
+        The API endpoint is determined in the following order:
+        (1) if `client_options.api_endpoint` if provided, use the provided one.
+        (2) if `GOOGLE_API_USE_CLIENT_CERTIFICATE` environment variable is "always", use the
+        default mTLS endpoint; if the environment variabel is "never", use the default API
+        endpoint; otherwise if client cert source exists, use the default mTLS endpoint, otherwise
+        use the default API endpoint.
+
+        More details can be found at https://google.aip.dev/auth/4114.
+
+        Args:
+            client_options (google.api_core.client_options.ClientOptions): Custom options for the
+                client. Only the `api_endpoint` and `client_cert_source` properties may be used
+                in this method.
+
+        Returns:
+            Tuple[str, Callable[[], Tuple[bytes, bytes]]]: returns the API endpoint and the
+                client cert source to use.
+
+        Raises:
+            google.auth.exceptions.MutualTLSChannelError: If any errors happen.
+        """
+        if client_options is None:
+            client_options = client_options_lib.ClientOptions()
+        use_client_cert = os.getenv("GOOGLE_API_USE_CLIENT_CERTIFICATE", "false")
+        use_mtls_endpoint = os.getenv("GOOGLE_API_USE_MTLS_ENDPOINT", "auto")
+        if use_client_cert not in ("true", "false"):
+            raise ValueError(
+                "Environment variable `GOOGLE_API_USE_CLIENT_CERTIFICATE` must be either `true` or `false`"
+            )
+        if use_mtls_endpoint not in ("auto", "never", "always"):
+            raise MutualTLSChannelError(
+                "Environment variable `GOOGLE_API_USE_MTLS_ENDPOINT` must be `never`, `auto` or `always`"
+            )
+
+        # Figure out the client cert source to use.
+        client_cert_source = None
+        if use_client_cert == "true":
+            if client_options.client_cert_source:
+                client_cert_source = client_options.client_cert_source
+            elif mtls.has_default_client_cert_source():
+                client_cert_source = mtls.default_client_cert_source()
+
+        # Figure out which api endpoint to use.
+        if client_options.api_endpoint is not None:
+            api_endpoint = client_options.api_endpoint
+        elif use_mtls_endpoint == "always" or (
+            use_mtls_endpoint == "auto" and client_cert_source
+        ):
+            api_endpoint = cls.DEFAULT_MTLS_ENDPOINT
+        else:
+            api_endpoint = cls.DEFAULT_ENDPOINT
+
+        return api_endpoint, client_cert_source
+
     def __init__(
         self,
         *,
@@ -288,50 +386,22 @@ class CatalogServiceClient(metaclass=CatalogServiceClientMeta):
         if client_options is None:
             client_options = client_options_lib.ClientOptions()
 
-        # Create SSL credentials for mutual TLS if needed.
-        use_client_cert = bool(
-            util.strtobool(os.getenv("GOOGLE_API_USE_CLIENT_CERTIFICATE", "false"))
+        api_endpoint, client_cert_source_func = self.get_mtls_endpoint_and_cert_source(
+            client_options
         )
 
-        client_cert_source_func = None
-        is_mtls = False
-        if use_client_cert:
-            if client_options.client_cert_source:
-                is_mtls = True
-                client_cert_source_func = client_options.client_cert_source
-            else:
-                is_mtls = mtls.has_default_client_cert_source()
-                if is_mtls:
-                    client_cert_source_func = mtls.default_client_cert_source()
-                else:
-                    client_cert_source_func = None
-
-        # Figure out which api endpoint to use.
-        if client_options.api_endpoint is not None:
-            api_endpoint = client_options.api_endpoint
-        else:
-            use_mtls_env = os.getenv("GOOGLE_API_USE_MTLS_ENDPOINT", "auto")
-            if use_mtls_env == "never":
-                api_endpoint = self.DEFAULT_ENDPOINT
-            elif use_mtls_env == "always":
-                api_endpoint = self.DEFAULT_MTLS_ENDPOINT
-            elif use_mtls_env == "auto":
-                if is_mtls:
-                    api_endpoint = self.DEFAULT_MTLS_ENDPOINT
-                else:
-                    api_endpoint = self.DEFAULT_ENDPOINT
-            else:
-                raise MutualTLSChannelError(
-                    "Unsupported GOOGLE_API_USE_MTLS_ENDPOINT value. Accepted "
-                    "values: never, auto, always"
-                )
+        api_key_value = getattr(client_options, "api_key", None)
+        if api_key_value and credentials:
+            raise ValueError(
+                "client_options.api_key and credentials are mutually exclusive"
+            )
 
         # Save or instantiate the transport.
         # Ordinarily, we provide the transport, but allowing a custom transport
         # instance provides an extensibility point for unusual situations.
         if isinstance(transport, CatalogServiceTransport):
             # transport is a CatalogServiceTransport instance.
-            if credentials or client_options.credentials_file:
+            if credentials or client_options.credentials_file or api_key_value:
                 raise ValueError(
                     "When providing a transport instance, "
                     "provide its credentials directly."
@@ -343,6 +413,15 @@ class CatalogServiceClient(metaclass=CatalogServiceClientMeta):
                 )
             self._transport = transport
         else:
+            import google.auth._default  # type: ignore
+
+            if api_key_value and hasattr(
+                google.auth._default, "get_api_key_credentials"
+            ):
+                credentials = google.auth._default.get_api_key_credentials(
+                    api_key_value
+                )
+
             Transport = type(self).get_transport_class(transport)
             self._transport = Transport(
                 credentials=credentials,
@@ -361,12 +440,37 @@ class CatalogServiceClient(metaclass=CatalogServiceClientMeta):
         *,
         parent: str = None,
         catalog_item: catalog.CatalogItem = None,
-        retry: retries.Retry = gapic_v1.method.DEFAULT,
+        retry: OptionalRetry = gapic_v1.method.DEFAULT,
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> catalog.CatalogItem:
         r"""Creates a catalog item.
 
+        .. code-block:: python
+
+            from google.cloud import recommendationengine_v1beta1
+
+            def sample_create_catalog_item():
+                # Create a client
+                client = recommendationengine_v1beta1.CatalogServiceClient()
+
+                # Initialize request argument(s)
+                catalog_item = recommendationengine_v1beta1.CatalogItem()
+                catalog_item.id = "id_value"
+                catalog_item.category_hierarchies.categories = ['categories_value_1', 'categories_value_2']
+                catalog_item.title = "title_value"
+
+                request = recommendationengine_v1beta1.CreateCatalogItemRequest(
+                    parent="parent_value",
+                    catalog_item=catalog_item,
+                )
+
+                # Make the request
+                response = client.create_catalog_item(request=request)
+
+                # Handle the response
+                print(response)
+
         Args:
             request (Union[google.cloud.recommendationengine_v1beta1.types.CreateCatalogItemRequest, dict]):
                 The request object. Request message for
@@ -396,7 +500,7 @@ class CatalogServiceClient(metaclass=CatalogServiceClientMeta):
 
         """
         # Create or coerce a protobuf request object.
-        # Sanity check: If we got a request object, we should *not* have
+        # Quick check: If we got a request object, we should *not* have
         # gotten any keyword arguments that map to the request.
         has_flattened_params = any([parent, catalog_item])
         if request is not None and has_flattened_params:
@@ -429,7 +533,12 @@ class CatalogServiceClient(metaclass=CatalogServiceClientMeta):
         )
 
         # Send the request.
-        response = rpc(request, retry=retry, timeout=timeout, metadata=metadata,)
+        response = rpc(
+            request,
+            retry=retry,
+            timeout=timeout,
+            metadata=metadata,
+        )
 
         # Done; return the response.
         return response
@@ -439,12 +548,31 @@ class CatalogServiceClient(metaclass=CatalogServiceClientMeta):
         request: Union[catalog_service.GetCatalogItemRequest, dict] = None,
         *,
         name: str = None,
-        retry: retries.Retry = gapic_v1.method.DEFAULT,
+        retry: OptionalRetry = gapic_v1.method.DEFAULT,
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> catalog.CatalogItem:
         r"""Gets a specific catalog item.
 
+        .. code-block:: python
+
+            from google.cloud import recommendationengine_v1beta1
+
+            def sample_get_catalog_item():
+                # Create a client
+                client = recommendationengine_v1beta1.CatalogServiceClient()
+
+                # Initialize request argument(s)
+                request = recommendationengine_v1beta1.GetCatalogItemRequest(
+                    name="name_value",
+                )
+
+                # Make the request
+                response = client.get_catalog_item(request=request)
+
+                # Handle the response
+                print(response)
+
         Args:
             request (Union[google.cloud.recommendationengine_v1beta1.types.GetCatalogItemRequest, dict]):
                 The request object. Request message for GetCatalogItem
@@ -469,7 +597,7 @@ class CatalogServiceClient(metaclass=CatalogServiceClientMeta):
 
         """
         # Create or coerce a protobuf request object.
-        # Sanity check: If we got a request object, we should *not* have
+        # Quick check: If we got a request object, we should *not* have
         # gotten any keyword arguments that map to the request.
         has_flattened_params = any([name])
         if request is not None and has_flattened_params:
@@ -500,7 +628,12 @@ class CatalogServiceClient(metaclass=CatalogServiceClientMeta):
         )
 
         # Send the request.
-        response = rpc(request, retry=retry, timeout=timeout, metadata=metadata,)
+        response = rpc(
+            request,
+            retry=retry,
+            timeout=timeout,
+            metadata=metadata,
+        )
 
         # Done; return the response.
         return response
@@ -511,12 +644,32 @@ class CatalogServiceClient(metaclass=CatalogServiceClientMeta):
         *,
         parent: str = None,
         filter: str = None,
-        retry: retries.Retry = gapic_v1.method.DEFAULT,
+        retry: OptionalRetry = gapic_v1.method.DEFAULT,
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> pagers.ListCatalogItemsPager:
         r"""Gets a list of catalog items.
 
+        .. code-block:: python
+
+            from google.cloud import recommendationengine_v1beta1
+
+            def sample_list_catalog_items():
+                # Create a client
+                client = recommendationengine_v1beta1.CatalogServiceClient()
+
+                # Initialize request argument(s)
+                request = recommendationengine_v1beta1.ListCatalogItemsRequest(
+                    parent="parent_value",
+                )
+
+                # Make the request
+                page_result = client.list_catalog_items(request=request)
+
+                # Handle the response
+                for response in page_result:
+                    print(response)
+
         Args:
             request (Union[google.cloud.recommendationengine_v1beta1.types.ListCatalogItemsRequest, dict]):
                 The request object. Request message for ListCatalogItems
@@ -551,7 +704,7 @@ class CatalogServiceClient(metaclass=CatalogServiceClientMeta):
 
         """
         # Create or coerce a protobuf request object.
-        # Sanity check: If we got a request object, we should *not* have
+        # Quick check: If we got a request object, we should *not* have
         # gotten any keyword arguments that map to the request.
         has_flattened_params = any([parent, filter])
         if request is not None and has_flattened_params:
@@ -584,12 +737,20 @@ class CatalogServiceClient(metaclass=CatalogServiceClientMeta):
         )
 
         # Send the request.
-        response = rpc(request, retry=retry, timeout=timeout, metadata=metadata,)
+        response = rpc(
+            request,
+            retry=retry,
+            timeout=timeout,
+            metadata=metadata,
+        )
 
         # This method is paged; wrap the response in a pager, which provides
         # an `__iter__` convenience method.
         response = pagers.ListCatalogItemsPager(
-            method=rpc, request=request, response=response, metadata=metadata,
+            method=rpc,
+            request=request,
+            response=response,
+            metadata=metadata,
         )
 
         # Done; return the response.
@@ -602,13 +763,38 @@ class CatalogServiceClient(metaclass=CatalogServiceClientMeta):
         name: str = None,
         catalog_item: catalog.CatalogItem = None,
         update_mask: field_mask_pb2.FieldMask = None,
-        retry: retries.Retry = gapic_v1.method.DEFAULT,
+        retry: OptionalRetry = gapic_v1.method.DEFAULT,
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> catalog.CatalogItem:
         r"""Updates a catalog item. Partial updating is
         supported. Non-existing items will be created.
 
+        .. code-block:: python
+
+            from google.cloud import recommendationengine_v1beta1
+
+            def sample_update_catalog_item():
+                # Create a client
+                client = recommendationengine_v1beta1.CatalogServiceClient()
+
+                # Initialize request argument(s)
+                catalog_item = recommendationengine_v1beta1.CatalogItem()
+                catalog_item.id = "id_value"
+                catalog_item.category_hierarchies.categories = ['categories_value_1', 'categories_value_2']
+                catalog_item.title = "title_value"
+
+                request = recommendationengine_v1beta1.UpdateCatalogItemRequest(
+                    name="name_value",
+                    catalog_item=catalog_item,
+                )
+
+                # Make the request
+                response = client.update_catalog_item(request=request)
+
+                # Handle the response
+                print(response)
+
         Args:
             request (Union[google.cloud.recommendationengine_v1beta1.types.UpdateCatalogItemRequest, dict]):
                 The request object. Request message for
@@ -648,7 +834,7 @@ class CatalogServiceClient(metaclass=CatalogServiceClientMeta):
 
         """
         # Create or coerce a protobuf request object.
-        # Sanity check: If we got a request object, we should *not* have
+        # Quick check: If we got a request object, we should *not* have
         # gotten any keyword arguments that map to the request.
         has_flattened_params = any([name, catalog_item, update_mask])
         if request is not None and has_flattened_params:
@@ -683,7 +869,12 @@ class CatalogServiceClient(metaclass=CatalogServiceClientMeta):
         )
 
         # Send the request.
-        response = rpc(request, retry=retry, timeout=timeout, metadata=metadata,)
+        response = rpc(
+            request,
+            retry=retry,
+            timeout=timeout,
+            metadata=metadata,
+        )
 
         # Done; return the response.
         return response
@@ -693,12 +884,28 @@ class CatalogServiceClient(metaclass=CatalogServiceClientMeta):
         request: Union[catalog_service.DeleteCatalogItemRequest, dict] = None,
         *,
         name: str = None,
-        retry: retries.Retry = gapic_v1.method.DEFAULT,
+        retry: OptionalRetry = gapic_v1.method.DEFAULT,
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> None:
         r"""Deletes a catalog item.
 
+        .. code-block:: python
+
+            from google.cloud import recommendationengine_v1beta1
+
+            def sample_delete_catalog_item():
+                # Create a client
+                client = recommendationengine_v1beta1.CatalogServiceClient()
+
+                # Initialize request argument(s)
+                request = recommendationengine_v1beta1.DeleteCatalogItemRequest(
+                    name="name_value",
+                )
+
+                # Make the request
+                client.delete_catalog_item(request=request)
+
         Args:
             request (Union[google.cloud.recommendationengine_v1beta1.types.DeleteCatalogItemRequest, dict]):
                 The request object. Request message for
@@ -717,7 +924,7 @@ class CatalogServiceClient(metaclass=CatalogServiceClientMeta):
                 sent along with the request as metadata.
         """
         # Create or coerce a protobuf request object.
-        # Sanity check: If we got a request object, we should *not* have
+        # Quick check: If we got a request object, we should *not* have
         # gotten any keyword arguments that map to the request.
         has_flattened_params = any([name])
         if request is not None and has_flattened_params:
@@ -749,7 +956,10 @@ class CatalogServiceClient(metaclass=CatalogServiceClientMeta):
 
         # Send the request.
         rpc(
-            request, retry=retry, timeout=timeout, metadata=metadata,
+            request,
+            retry=retry,
+            timeout=timeout,
+            metadata=metadata,
         )
 
     def import_catalog_items(
@@ -760,7 +970,7 @@ class CatalogServiceClient(metaclass=CatalogServiceClientMeta):
         request_id: str = None,
         input_config: import_.InputConfig = None,
         errors_config: import_.ImportErrorsConfig = None,
-        retry: retries.Retry = gapic_v1.method.DEFAULT,
+        retry: OptionalRetry = gapic_v1.method.DEFAULT,
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> operation.Operation:
@@ -772,6 +982,29 @@ class CatalogServiceClient(metaclass=CatalogServiceClientMeta):
         it is possible for a subset of the items to be
         successfully updated.
 
+        .. code-block:: python
+
+            from google.cloud import recommendationengine_v1beta1
+
+            def sample_import_catalog_items():
+                # Create a client
+                client = recommendationengine_v1beta1.CatalogServiceClient()
+
+                # Initialize request argument(s)
+                request = recommendationengine_v1beta1.ImportCatalogItemsRequest(
+                    parent="parent_value",
+                )
+
+                # Make the request
+                operation = client.import_catalog_items(request=request)
+
+                print("Waiting for operation to complete...")
+
+                response = operation.result()
+
+                # Handle the response
+                print(response)
+
         Args:
             request (Union[google.cloud.recommendationengine_v1beta1.types.ImportCatalogItemsRequest, dict]):
                 The request object. Request message for Import methods.
@@ -826,7 +1059,7 @@ class CatalogServiceClient(metaclass=CatalogServiceClientMeta):
 
         """
         # Create or coerce a protobuf request object.
-        # Sanity check: If we got a request object, we should *not* have
+        # Quick check: If we got a request object, we should *not* have
         # gotten any keyword arguments that map to the request.
         has_flattened_params = any([parent, request_id, input_config, errors_config])
         if request is not None and has_flattened_params:
@@ -863,7 +1096,12 @@ class CatalogServiceClient(metaclass=CatalogServiceClientMeta):
         )
 
         # Send the request.
-        response = rpc(request, retry=retry, timeout=timeout, metadata=metadata,)
+        response = rpc(
+            request,
+            retry=retry,
+            timeout=timeout,
+            metadata=metadata,
+        )
 
         # Wrap the response in an operation future.
         response = operation.from_gapic(
