@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2020 Google LLC
+# Copyright 2022 Google LLC
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -16,16 +16,21 @@
 from collections import OrderedDict
 import functools
 import re
-from typing import Dict, Sequence, Tuple, Type, Union
+from typing import Dict, Mapping, Optional, Sequence, Tuple, Type, Union
 import pkg_resources
 
-import google.api_core.client_options as ClientOptions  # type: ignore
-from google.api_core import exceptions as core_exceptions  # type: ignore
-from google.api_core import gapic_v1  # type: ignore
-from google.api_core import retry as retries  # type: ignore
+from google.api_core.client_options import ClientOptions
+from google.api_core import exceptions as core_exceptions
+from google.api_core import gapic_v1
+from google.api_core import retry as retries
 from google.auth import credentials as ga_credentials  # type: ignore
 from google.oauth2 import service_account  # type: ignore
 
+try:
+    OptionalRetry = Union[retries.Retry, gapic_v1.method._MethodDefault]
+except AttributeError:  # pragma: NO COVER
+    OptionalRetry = Union[retries.Retry, object]  # type: ignore
+
 from google.cloud.recommendationengine_v1beta1.services.prediction_api_key_registry import (
     pagers,
 )
@@ -123,6 +128,42 @@ class PredictionApiKeyRegistryAsyncClient:
 
     from_service_account_json = from_service_account_file
 
+    @classmethod
+    def get_mtls_endpoint_and_cert_source(
+        cls, client_options: Optional[ClientOptions] = None
+    ):
+        """Return the API endpoint and client cert source for mutual TLS.
+
+        The client cert source is determined in the following order:
+        (1) if `GOOGLE_API_USE_CLIENT_CERTIFICATE` environment variable is not "true", the
+        client cert source is None.
+        (2) if `client_options.client_cert_source` is provided, use the provided one; if the
+        default client cert source exists, use the default one; otherwise the client cert
+        source is None.
+
+        The API endpoint is determined in the following order:
+        (1) if `client_options.api_endpoint` if provided, use the provided one.
+        (2) if `GOOGLE_API_USE_CLIENT_CERTIFICATE` environment variable is "always", use the
+        default mTLS endpoint; if the environment variabel is "never", use the default API
+        endpoint; otherwise if client cert source exists, use the default mTLS endpoint, otherwise
+        use the default API endpoint.
+
+        More details can be found at https://google.aip.dev/auth/4114.
+
+        Args:
+            client_options (google.api_core.client_options.ClientOptions): Custom options for the
+                client. Only the `api_endpoint` and `client_cert_source` properties may be used
+                in this method.
+
+        Returns:
+            Tuple[str, Callable[[], Tuple[bytes, bytes]]]: returns the API endpoint and the
+                client cert source to use.
+
+        Raises:
+            google.auth.exceptions.MutualTLSChannelError: If any errors happen.
+        """
+        return PredictionApiKeyRegistryClient.get_mtls_endpoint_and_cert_source(client_options)  # type: ignore
+
     @property
     def transport(self) -> PredictionApiKeyRegistryTransport:
         """Returns the transport used by the client instance.
@@ -186,18 +227,40 @@ class PredictionApiKeyRegistryAsyncClient:
 
     async def create_prediction_api_key_registration(
         self,
-        request: prediction_apikey_registry_service.CreatePredictionApiKeyRegistrationRequest = None,
+        request: Union[
+            prediction_apikey_registry_service.CreatePredictionApiKeyRegistrationRequest,
+            dict,
+        ] = None,
         *,
         parent: str = None,
         prediction_api_key_registration: prediction_apikey_registry_service.PredictionApiKeyRegistration = None,
-        retry: retries.Retry = gapic_v1.method.DEFAULT,
+        retry: OptionalRetry = gapic_v1.method.DEFAULT,
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> prediction_apikey_registry_service.PredictionApiKeyRegistration:
         r"""Register an API key for use with predict method.
 
+        .. code-block:: python
+
+            from google.cloud import recommendationengine_v1beta1
+
+            async def sample_create_prediction_api_key_registration():
+                # Create a client
+                client = recommendationengine_v1beta1.PredictionApiKeyRegistryAsyncClient()
+
+                # Initialize request argument(s)
+                request = recommendationengine_v1beta1.CreatePredictionApiKeyRegistrationRequest(
+                    parent="parent_value",
+                )
+
+                # Make the request
+                response = await client.create_prediction_api_key_registration(request=request)
+
+                # Handle the response
+                print(response)
+
         Args:
-            request (:class:`google.cloud.recommendationengine_v1beta1.types.CreatePredictionApiKeyRegistrationRequest`):
+            request (Union[google.cloud.recommendationengine_v1beta1.types.CreatePredictionApiKeyRegistrationRequest, dict]):
                 The request object. Request message for the
                 `CreatePredictionApiKeyRegistration` method.
             parent (:class:`str`):
@@ -225,7 +288,7 @@ class PredictionApiKeyRegistryAsyncClient:
                 Registered Api Key.
         """
         # Create or coerce a protobuf request object.
-        # Sanity check: If we got a request object, we should *not* have
+        # Quick check: If we got a request object, we should *not* have
         # gotten any keyword arguments that map to the request.
         has_flattened_params = any([parent, prediction_api_key_registration])
         if request is not None and has_flattened_params:
@@ -270,25 +333,53 @@ class PredictionApiKeyRegistryAsyncClient:
         )
 
         # Send the request.
-        response = await rpc(request, retry=retry, timeout=timeout, metadata=metadata,)
+        response = await rpc(
+            request,
+            retry=retry,
+            timeout=timeout,
+            metadata=metadata,
+        )
 
         # Done; return the response.
         return response
 
     async def list_prediction_api_key_registrations(
         self,
-        request: prediction_apikey_registry_service.ListPredictionApiKeyRegistrationsRequest = None,
+        request: Union[
+            prediction_apikey_registry_service.ListPredictionApiKeyRegistrationsRequest,
+            dict,
+        ] = None,
         *,
         parent: str = None,
-        retry: retries.Retry = gapic_v1.method.DEFAULT,
+        retry: OptionalRetry = gapic_v1.method.DEFAULT,
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> pagers.ListPredictionApiKeyRegistrationsAsyncPager:
         r"""List the registered apiKeys for use with predict
         method.
 
+        .. code-block:: python
+
+            from google.cloud import recommendationengine_v1beta1
+
+            async def sample_list_prediction_api_key_registrations():
+                # Create a client
+                client = recommendationengine_v1beta1.PredictionApiKeyRegistryAsyncClient()
+
+                # Initialize request argument(s)
+                request = recommendationengine_v1beta1.ListPredictionApiKeyRegistrationsRequest(
+                    parent="parent_value",
+                )
+
+                # Make the request
+                page_result = client.list_prediction_api_key_registrations(request=request)
+
+                # Handle the response
+                async for response in page_result:
+                    print(response)
+
         Args:
-            request (:class:`google.cloud.recommendationengine_v1beta1.types.ListPredictionApiKeyRegistrationsRequest`):
+            request (Union[google.cloud.recommendationengine_v1beta1.types.ListPredictionApiKeyRegistrationsRequest, dict]):
                 The request object. Request message for the
                 `ListPredictionApiKeyRegistrations`.
             parent (:class:`str`):
@@ -314,7 +405,7 @@ class PredictionApiKeyRegistryAsyncClient:
 
         """
         # Create or coerce a protobuf request object.
-        # Sanity check: If we got a request object, we should *not* have
+        # Quick check: If we got a request object, we should *not* have
         # gotten any keyword arguments that map to the request.
         has_flattened_params = any([parent])
         if request is not None and has_flattened_params:
@@ -323,8 +414,10 @@ class PredictionApiKeyRegistryAsyncClient:
                 "the individual field arguments should be set."
             )
 
-        request = prediction_apikey_registry_service.ListPredictionApiKeyRegistrationsRequest(
-            request
+        request = (
+            prediction_apikey_registry_service.ListPredictionApiKeyRegistrationsRequest(
+                request
+            )
         )
 
         # If we have keyword arguments corresponding to fields on the
@@ -357,12 +450,20 @@ class PredictionApiKeyRegistryAsyncClient:
         )
 
         # Send the request.
-        response = await rpc(request, retry=retry, timeout=timeout, metadata=metadata,)
+        response = await rpc(
+            request,
+            retry=retry,
+            timeout=timeout,
+            metadata=metadata,
+        )
 
         # This method is paged; wrap the response in a pager, which provides
         # an `__aiter__` convenience method.
         response = pagers.ListPredictionApiKeyRegistrationsAsyncPager(
-            method=rpc, request=request, response=response, metadata=metadata,
+            method=rpc,
+            request=request,
+            response=response,
+            metadata=metadata,
         )
 
         # Done; return the response.
@@ -370,17 +471,36 @@ class PredictionApiKeyRegistryAsyncClient:
 
     async def delete_prediction_api_key_registration(
         self,
-        request: prediction_apikey_registry_service.DeletePredictionApiKeyRegistrationRequest = None,
+        request: Union[
+            prediction_apikey_registry_service.DeletePredictionApiKeyRegistrationRequest,
+            dict,
+        ] = None,
         *,
         name: str = None,
-        retry: retries.Retry = gapic_v1.method.DEFAULT,
+        retry: OptionalRetry = gapic_v1.method.DEFAULT,
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> None:
         r"""Unregister an apiKey from using for predict method.
 
+        .. code-block:: python
+
+            from google.cloud import recommendationengine_v1beta1
+
+            async def sample_delete_prediction_api_key_registration():
+                # Create a client
+                client = recommendationengine_v1beta1.PredictionApiKeyRegistryAsyncClient()
+
+                # Initialize request argument(s)
+                request = recommendationengine_v1beta1.DeletePredictionApiKeyRegistrationRequest(
+                    name="name_value",
+                )
+
+                # Make the request
+                await client.delete_prediction_api_key_registration(request=request)
+
         Args:
-            request (:class:`google.cloud.recommendationengine_v1beta1.types.DeletePredictionApiKeyRegistrationRequest`):
+            request (Union[google.cloud.recommendationengine_v1beta1.types.DeletePredictionApiKeyRegistrationRequest, dict]):
                 The request object. Request message for
                 `DeletePredictionApiKeyRegistration` method.
             name (:class:`str`):
@@ -398,7 +518,7 @@ class PredictionApiKeyRegistryAsyncClient:
                 sent along with the request as metadata.
         """
         # Create or coerce a protobuf request object.
-        # Sanity check: If we got a request object, we should *not* have
+        # Quick check: If we got a request object, we should *not* have
         # gotten any keyword arguments that map to the request.
         has_flattened_params = any([name])
         if request is not None and has_flattened_params:
@@ -442,7 +562,10 @@ class PredictionApiKeyRegistryAsyncClient:
 
         # Send the request.
         await rpc(
-            request, retry=retry, timeout=timeout, metadata=metadata,
+            request,
+            retry=retry,
+            timeout=timeout,
+            metadata=metadata,
         )
 
     async def __aenter__(self):
