@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2020 Google LLC
+# Copyright 2022 Google LLC
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -16,16 +16,21 @@
 from collections import OrderedDict
 import functools
 import re
-from typing import Dict, Sequence, Tuple, Type, Union
+from typing import Dict, Mapping, Optional, Sequence, Tuple, Type, Union
 import pkg_resources
 
-import google.api_core.client_options as ClientOptions  # type: ignore
-from google.api_core import exceptions as core_exceptions  # type: ignore
-from google.api_core import gapic_v1  # type: ignore
-from google.api_core import retry as retries  # type: ignore
+from google.api_core.client_options import ClientOptions
+from google.api_core import exceptions as core_exceptions
+from google.api_core import gapic_v1
+from google.api_core import retry as retries
 from google.auth import credentials as ga_credentials  # type: ignore
 from google.oauth2 import service_account  # type: ignore
 
+try:
+    OptionalRetry = Union[retries.Retry, gapic_v1.method._MethodDefault]
+except AttributeError:  # pragma: NO COVER
+    OptionalRetry = Union[retries.Retry, object]  # type: ignore
+
 from google.api import httpbody_pb2  # type: ignore
 from google.api_core import operation  # type: ignore
 from google.api_core import operation_async  # type: ignore
@@ -111,6 +116,42 @@ class UserEventServiceAsyncClient:
 
     from_service_account_json = from_service_account_file
 
+    @classmethod
+    def get_mtls_endpoint_and_cert_source(
+        cls, client_options: Optional[ClientOptions] = None
+    ):
+        """Return the API endpoint and client cert source for mutual TLS.
+
+        The client cert source is determined in the following order:
+        (1) if `GOOGLE_API_USE_CLIENT_CERTIFICATE` environment variable is not "true", the
+        client cert source is None.
+        (2) if `client_options.client_cert_source` is provided, use the provided one; if the
+        default client cert source exists, use the default one; otherwise the client cert
+        source is None.
+
+        The API endpoint is determined in the following order:
+        (1) if `client_options.api_endpoint` if provided, use the provided one.
+        (2) if `GOOGLE_API_USE_CLIENT_CERTIFICATE` environment variable is "always", use the
+        default mTLS endpoint; if the environment variabel is "never", use the default API
+        endpoint; otherwise if client cert source exists, use the default mTLS endpoint, otherwise
+        use the default API endpoint.
+
+        More details can be found at https://google.aip.dev/auth/4114.
+
+        Args:
+            client_options (google.api_core.client_options.ClientOptions): Custom options for the
+                client. Only the `api_endpoint` and `client_cert_source` properties may be used
+                in this method.
+
+        Returns:
+            Tuple[str, Callable[[], Tuple[bytes, bytes]]]: returns the API endpoint and the
+                client cert source to use.
+
+        Raises:
+            google.auth.exceptions.MutualTLSChannelError: If any errors happen.
+        """
+        return UserEventServiceClient.get_mtls_endpoint_and_cert_source(client_options)  # type: ignore
+
     @property
     def transport(self) -> UserEventServiceTransport:
         """Returns the transport used by the client instance.
@@ -173,18 +214,42 @@ class UserEventServiceAsyncClient:
 
     async def write_user_event(
         self,
-        request: user_event_service.WriteUserEventRequest = None,
+        request: Union[user_event_service.WriteUserEventRequest, dict] = None,
         *,
         parent: str = None,
         user_event: gcr_user_event.UserEvent = None,
-        retry: retries.Retry = gapic_v1.method.DEFAULT,
+        retry: OptionalRetry = gapic_v1.method.DEFAULT,
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> gcr_user_event.UserEvent:
         r"""Writes a single user event.
 
+        .. code-block:: python
+
+            from google.cloud import recommendationengine_v1beta1
+
+            async def sample_write_user_event():
+                # Create a client
+                client = recommendationengine_v1beta1.UserEventServiceAsyncClient()
+
+                # Initialize request argument(s)
+                user_event = recommendationengine_v1beta1.UserEvent()
+                user_event.event_type = "event_type_value"
+                user_event.user_info.visitor_id = "visitor_id_value"
+
+                request = recommendationengine_v1beta1.WriteUserEventRequest(
+                    parent="parent_value",
+                    user_event=user_event,
+                )
+
+                # Make the request
+                response = await client.write_user_event(request=request)
+
+                # Handle the response
+                print(response)
+
         Args:
-            request (:class:`google.cloud.recommendationengine_v1beta1.types.WriteUserEventRequest`):
+            request (Union[google.cloud.recommendationengine_v1beta1.types.WriteUserEventRequest, dict]):
                 The request object. Request message for WriteUserEvent
                 method.
             parent (:class:`str`):
@@ -214,7 +279,7 @@ class UserEventServiceAsyncClient:
 
         """
         # Create or coerce a protobuf request object.
-        # Sanity check: If we got a request object, we should *not* have
+        # Quick check: If we got a request object, we should *not* have
         # gotten any keyword arguments that map to the request.
         has_flattened_params = any([parent, user_event])
         if request is not None and has_flattened_params:
@@ -257,20 +322,25 @@ class UserEventServiceAsyncClient:
         )
 
         # Send the request.
-        response = await rpc(request, retry=retry, timeout=timeout, metadata=metadata,)
+        response = await rpc(
+            request,
+            retry=retry,
+            timeout=timeout,
+            metadata=metadata,
+        )
 
         # Done; return the response.
         return response
 
     async def collect_user_event(
         self,
-        request: user_event_service.CollectUserEventRequest = None,
+        request: Union[user_event_service.CollectUserEventRequest, dict] = None,
         *,
         parent: str = None,
         user_event: str = None,
         uri: str = None,
         ets: int = None,
-        retry: retries.Retry = gapic_v1.method.DEFAULT,
+        retry: OptionalRetry = gapic_v1.method.DEFAULT,
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> httpbody_pb2.HttpBody:
@@ -281,8 +351,28 @@ class UserEventServiceAsyncClient:
         JavaScript pixel. Users should not call this method
         directly.
 
+        .. code-block:: python
+
+            from google.cloud import recommendationengine_v1beta1
+
+            async def sample_collect_user_event():
+                # Create a client
+                client = recommendationengine_v1beta1.UserEventServiceAsyncClient()
+
+                # Initialize request argument(s)
+                request = recommendationengine_v1beta1.CollectUserEventRequest(
+                    parent="parent_value",
+                    user_event="user_event_value",
+                )
+
+                # Make the request
+                response = await client.collect_user_event(request=request)
+
+                # Handle the response
+                print(response)
+
         Args:
-            request (:class:`google.cloud.recommendationengine_v1beta1.types.CollectUserEventRequest`):
+            request (Union[google.cloud.recommendationengine_v1beta1.types.CollectUserEventRequest, dict]):
                 The request object. Request message for CollectUserEvent
                 method.
             parent (:class:`str`):
@@ -300,8 +390,8 @@ class UserEventServiceAsyncClient:
                 on the ``request`` instance; if ``request`` is provided, this
                 should not be set.
             uri (:class:`str`):
-                Optional. The url including cgi-
-                arameters but excluding the hash
+                Optional. The url including
+                cgi-parameters but excluding the hash
                 fragment. The URL must be truncated to
                 1.5K bytes to conservatively be under
                 the 2K bytes. This is often more useful
@@ -379,7 +469,7 @@ class UserEventServiceAsyncClient:
 
         """
         # Create or coerce a protobuf request object.
-        # Sanity check: If we got a request object, we should *not* have
+        # Quick check: If we got a request object, we should *not* have
         # gotten any keyword arguments that map to the request.
         has_flattened_params = any([parent, user_event, uri, ets])
         if request is not None and has_flattened_params:
@@ -426,26 +516,51 @@ class UserEventServiceAsyncClient:
         )
 
         # Send the request.
-        response = await rpc(request, retry=retry, timeout=timeout, metadata=metadata,)
+        response = await rpc(
+            request,
+            retry=retry,
+            timeout=timeout,
+            metadata=metadata,
+        )
 
         # Done; return the response.
         return response
 
     async def list_user_events(
         self,
-        request: user_event_service.ListUserEventsRequest = None,
+        request: Union[user_event_service.ListUserEventsRequest, dict] = None,
         *,
         parent: str = None,
         filter: str = None,
-        retry: retries.Retry = gapic_v1.method.DEFAULT,
+        retry: OptionalRetry = gapic_v1.method.DEFAULT,
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> pagers.ListUserEventsAsyncPager:
         r"""Gets a list of user events within a time range, with
         potential filtering.
 
+        .. code-block:: python
+
+            from google.cloud import recommendationengine_v1beta1
+
+            async def sample_list_user_events():
+                # Create a client
+                client = recommendationengine_v1beta1.UserEventServiceAsyncClient()
+
+                # Initialize request argument(s)
+                request = recommendationengine_v1beta1.ListUserEventsRequest(
+                    parent="parent_value",
+                )
+
+                # Make the request
+                page_result = client.list_user_events(request=request)
+
+                # Handle the response
+                async for response in page_result:
+                    print(response)
+
         Args:
-            request (:class:`google.cloud.recommendationengine_v1beta1.types.ListUserEventsRequest`):
+            request (Union[google.cloud.recommendationengine_v1beta1.types.ListUserEventsRequest, dict]):
                 The request object. Request message for ListUserEvents
                 method.
             parent (:class:`str`):
@@ -512,7 +627,7 @@ class UserEventServiceAsyncClient:
 
         """
         # Create or coerce a protobuf request object.
-        # Sanity check: If we got a request object, we should *not* have
+        # Quick check: If we got a request object, we should *not* have
         # gotten any keyword arguments that map to the request.
         has_flattened_params = any([parent, filter])
         if request is not None and has_flattened_params:
@@ -555,12 +670,20 @@ class UserEventServiceAsyncClient:
         )
 
         # Send the request.
-        response = await rpc(request, retry=retry, timeout=timeout, metadata=metadata,)
+        response = await rpc(
+            request,
+            retry=retry,
+            timeout=timeout,
+            metadata=metadata,
+        )
 
         # This method is paged; wrap the response in a pager, which provides
         # an `__aiter__` convenience method.
         response = pagers.ListUserEventsAsyncPager(
-            method=rpc, request=request, response=response, metadata=metadata,
+            method=rpc,
+            request=request,
+            response=response,
+            metadata=metadata,
         )
 
         # Done; return the response.
@@ -568,12 +691,12 @@ class UserEventServiceAsyncClient:
 
     async def purge_user_events(
         self,
-        request: user_event_service.PurgeUserEventsRequest = None,
+        request: Union[user_event_service.PurgeUserEventsRequest, dict] = None,
         *,
         parent: str = None,
         filter: str = None,
         force: bool = None,
-        retry: retries.Retry = gapic_v1.method.DEFAULT,
+        retry: OptionalRetry = gapic_v1.method.DEFAULT,
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> operation_async.AsyncOperation:
@@ -583,8 +706,32 @@ class UserEventServiceAsyncClient:
         or days to complete. To test a filter, use the list
         command first.
 
+        .. code-block:: python
+
+            from google.cloud import recommendationengine_v1beta1
+
+            async def sample_purge_user_events():
+                # Create a client
+                client = recommendationengine_v1beta1.UserEventServiceAsyncClient()
+
+                # Initialize request argument(s)
+                request = recommendationengine_v1beta1.PurgeUserEventsRequest(
+                    parent="parent_value",
+                    filter="filter_value",
+                )
+
+                # Make the request
+                operation = client.purge_user_events(request=request)
+
+                print("Waiting for operation to complete...")
+
+                response = await operation.result()
+
+                # Handle the response
+                print(response)
+
         Args:
-            request (:class:`google.cloud.recommendationengine_v1beta1.types.PurgeUserEventsRequest`):
+            request (Union[google.cloud.recommendationengine_v1beta1.types.PurgeUserEventsRequest, dict]):
                 The request object. Request message for PurgeUserEvents
                 method.
             parent (:class:`str`):
@@ -645,7 +792,7 @@ class UserEventServiceAsyncClient:
 
         """
         # Create or coerce a protobuf request object.
-        # Sanity check: If we got a request object, we should *not* have
+        # Quick check: If we got a request object, we should *not* have
         # gotten any keyword arguments that map to the request.
         has_flattened_params = any([parent, filter, force])
         if request is not None and has_flattened_params:
@@ -690,7 +837,12 @@ class UserEventServiceAsyncClient:
         )
 
         # Send the request.
-        response = await rpc(request, retry=retry, timeout=timeout, metadata=metadata,)
+        response = await rpc(
+            request,
+            retry=retry,
+            timeout=timeout,
+            metadata=metadata,
+        )
 
         # Wrap the response in an operation future.
         response = operation_async.from_gapic(
@@ -705,13 +857,13 @@ class UserEventServiceAsyncClient:
 
     async def import_user_events(
         self,
-        request: import_.ImportUserEventsRequest = None,
+        request: Union[import_.ImportUserEventsRequest, dict] = None,
         *,
         parent: str = None,
         request_id: str = None,
         input_config: import_.InputConfig = None,
         errors_config: import_.ImportErrorsConfig = None,
-        retry: retries.Retry = gapic_v1.method.DEFAULT,
+        retry: OptionalRetry = gapic_v1.method.DEFAULT,
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> operation_async.AsyncOperation:
@@ -723,8 +875,31 @@ class UserEventServiceAsyncClient:
         successfully inserted. Operation.metadata is of type
         ImportMetadata.
 
+        .. code-block:: python
+
+            from google.cloud import recommendationengine_v1beta1
+
+            async def sample_import_user_events():
+                # Create a client
+                client = recommendationengine_v1beta1.UserEventServiceAsyncClient()
+
+                # Initialize request argument(s)
+                request = recommendationengine_v1beta1.ImportUserEventsRequest(
+                    parent="parent_value",
+                )
+
+                # Make the request
+                operation = client.import_user_events(request=request)
+
+                print("Waiting for operation to complete...")
+
+                response = await operation.result()
+
+                # Handle the response
+                print(response)
+
         Args:
-            request (:class:`google.cloud.recommendationengine_v1beta1.types.ImportUserEventsRequest`):
+            request (Union[google.cloud.recommendationengine_v1beta1.types.ImportUserEventsRequest, dict]):
                 The request object. Request message for the
                 ImportUserEvents request.
             parent (:class:`str`):
@@ -778,7 +953,7 @@ class UserEventServiceAsyncClient:
 
         """
         # Create or coerce a protobuf request object.
-        # Sanity check: If we got a request object, we should *not* have
+        # Quick check: If we got a request object, we should *not* have
         # gotten any keyword arguments that map to the request.
         has_flattened_params = any([parent, request_id, input_config, errors_config])
         if request is not None and has_flattened_params:
@@ -825,7 +1000,12 @@ class UserEventServiceAsyncClient:
         )
 
         # Send the request.
-        response = await rpc(request, retry=retry, timeout=timeout, metadata=metadata,)
+        response = await rpc(
+            request,
+            retry=retry,
+            timeout=timeout,
+            metadata=metadata,
+        )
 
         # Wrap the response in an operation future.
         response = operation_async.from_gapic(
